@@ -48,9 +48,9 @@
 #	getsampwidth()	-- returns sample width in bytes
 #	getframerate()	-- returns sampling frequency
 #	getnframes()	-- returns number of audio frames
-#	getcomptype()	-- returns compression type ('NONE' for AIFF files)
+#	getcomptype()	-- returns compression type ('NONE' or 'ULAW')
 #	getcompname()	-- returns human-readable version of
-#			   compression type ('not compressed' for AIFF files)
+#			   compression type ('not compressed' matches 'NONE')
 #	getparams()	-- returns a tuple consisting of all of the
 #			   above in the above order
 #	getmarkers()	-- returns None (for compatibility with the
@@ -82,8 +82,7 @@
 #	setcomptype(type, name)
 #			-- set the compression type and the
 #			   human-readable compression type
-#	setparams(nchannels, sampwidth, framerate, nframes, comptype, compname)
-#			-- set all parameters at once
+#	setparams(tuple)-- set all parameters at once
 #	tell()		-- return current position in output file
 #	writeframesraw(data)
 #			-- write audio frames without pathing up the
@@ -149,14 +148,11 @@ def _write_u32(file, x):
 		file.write(chr(int(data[i])))
 
 class Au_read:
-	access _file, _soundpos, _hdr_size, _data_size, _encoding, \
-		  _sampwidth, _framesize, _framerate, _nchannels, \
-		  _info: private
 
 	def __init__(self, f):
 		if type(f) == type(''):
 			import __builtin__
-			f = __builtin__.open(f, 'r')
+			f = __builtin__.open(f, 'rb')
 		self.initfp(f)
 
 	def __del__(self):
@@ -181,10 +177,11 @@ class Au_read:
 		if self._encoding not in _simple_encodings:
 			raise Error, 'encoding not (yet) supported'
 		if self._encoding in (AUDIO_FILE_ENCODING_MULAW_8,
-			  AUDIO_FILE_ENCODING_LINEAR_8,
 			  AUDIO_FILE_ENCODING_ALAW_8):
 			self._sampwidth = 2
 			self._framesize = 1
+		elif self._encoding == AUDIO_FILE_ENCODING_LINEAR_8:
+			self._framesize = self._sampwidth = 1
 		elif self._encoding == AUDIO_FILE_ENCODING_LINEAR_16:
 			self._framesize = self._sampwidth = 2
 		elif self._encoding == AUDIO_FILE_ENCODING_LINEAR_24:
@@ -280,14 +277,11 @@ class Au_read:
 		self._file = None
 
 class Au_write:
-	access _file, _framerate, _nchannels, _sampwidth, _framesize, \
-		  _nframes, _nframeswritten, _datawritten, _info, \
-		  _comptype: private
 
 	def __init__(self, f):
 		if type(f) == type(''):
 			import __builtin__
-			f = __builtin__.open(f, 'w')
+			f = __builtin__.open(f, 'wb')
 		self.initfp(f)
 
 	def __del__(self):
@@ -410,7 +404,6 @@ class Au_write:
 	#
 	# private methods
 	#
-	if 0: access *: private
 
 	def _ensure_header_written(self):
 		if not self._nframeswritten:
