@@ -122,6 +122,7 @@ None              clear()
 None              clrtobot()
 None              clrtoeol()
 None              scroll()
+                  scroll(nlines)
 None              touchwin()
 None              touchline(start,count)
 IntObject         getch(y,x)
@@ -168,9 +169,13 @@ typedef struct {
 	WINDOW *pad;
 } PyCursesPadObject;
 
+#if 0
 staticforward PyTypeObject PyCursesScreen_Type;
+#endif
 staticforward PyTypeObject PyCursesWindow_Type;
+#if 0
 staticforward PyTypeObject PyCursesPad_Type;
+#endif
 
 #define PyCursesScreen_Check(v)	 ((v)->ob_type == &PyCursesScreen_Type)
 #define PyCursesWindow_Check(v)	 ((v)->ob_type == &PyCursesWindow_Type)
@@ -215,7 +220,7 @@ Version 1.0: 94/08/30:
 
 ******************************************************************/
 
-char *PyCursesVersion = "1.2";
+static char *PyCursesVersion = "1.2";
 
 /*
  * Check the return code from a curses function and return None 
@@ -386,7 +391,7 @@ PyCursesWindow_AddCh(self,arg)
   int rtn;
   int x, y;
   int ch;
-  int attr, attr_old;
+  int attr, attr_old = 0;
   int use_xy = FALSE, use_attr = FALSE;
 
   switch (ARG_COUNT(arg)) {
@@ -436,7 +441,7 @@ PyCursesWindow_InsCh(self,arg)
   int rtn;
   int x, y;
   int ch;
-  int attr, attr_old;
+  int attr, attr_old = 0;
   int use_xy = TRUE, use_attr = FALSE;
 
   switch (ARG_COUNT(arg)) {
@@ -542,7 +547,7 @@ PyCursesWindow_AddStr(self,arg)
   int rtn;
   int x, y;
   char *str;
-  int attr, attr_old;
+  int attr, attr_old = 0;
   int use_xy = FALSE, use_attr = FALSE;
 
   switch (ARG_COUNT(arg)) {
@@ -837,9 +842,24 @@ PyCursesWindow_Scroll(self,arg)
      PyCursesWindowObject *self;
      PyObject * arg;
 {
-  if (!PyArg_NoArgs(arg))
+  int nlines;
+  int use_nlines = FALSE;
+  switch (ARG_COUNT(arg)) {
+  case 0:
+    break;
+  case 1:
+    if (!PyArg_Parse(arg, "i;nlines", &nlines))
       return NULL;
-  return PyCursesCheckERR(scroll(self->win), "scroll");
+    use_nlines = TRUE;
+    break;
+  default:
+    PyErr_SetString(PyExc_TypeError, "scroll requires 0 or 1 arguments");
+    return NULL;
+  }
+  if (use_nlines)
+    return PyCursesCheckERR(wscrl(self->win, nlines), "scroll");
+  else
+    return PyCursesCheckERR(scroll(self->win), "scroll");
 }
 
 static PyObject *
@@ -1116,6 +1136,7 @@ PyCursesPad_New(pad)
 
 /* -------------------------------------------------------*/
 
+#if 0
 static PyTypeObject PyCursesScreen_Type = {
 	PyObject_HEAD_INIT(&PyType_Type)
 	0,			/*ob_size*/
@@ -1134,6 +1155,7 @@ static PyTypeObject PyCursesScreen_Type = {
 	0,			/*tp_as_mapping*/
 	0,			/*tp_hash*/
 };
+#endif
 
 static PyTypeObject PyCursesWindow_Type = {
 	PyObject_HEAD_INIT(&PyType_Type)
@@ -1154,6 +1176,7 @@ static PyTypeObject PyCursesWindow_Type = {
 	0,			/*tp_hash*/
 };
 
+#if 0
 static PyTypeObject PyCursesPad_Type = {
 	PyObject_HEAD_INIT(&PyType_Type)
 	0,			/*ob_size*/
@@ -1172,6 +1195,7 @@ static PyTypeObject PyCursesPad_Type = {
 	0,			/*tp_as_mapping*/
 	0,			/*tp_hash*/
 };
+#endif
 
 
 /* -------------------------------------------------------*/
@@ -1224,7 +1248,8 @@ PyCurses_InitScr(self, args)
 	SetDictInt("ACS_DEGREE",  (ACS_DEGREE));
 	SetDictInt("ACS_PLMINUS", (ACS_PLMINUS));
 	SetDictInt("ACS_BULLET",  (ACS_BULLET));
-	SetDictInt("ACS_LARROW",  (ACS_RARROW));
+	SetDictInt("ACS_LARROW",  (ACS_LARROW));
+	SetDictInt("ACS_RARROW",  (ACS_RARROW));
 	SetDictInt("ACS_DARROW",  (ACS_DARROW));
 	SetDictInt("ACS_UARROW",  (ACS_UARROW));
 	SetDictInt("ACS_BOARD",   (ACS_BOARD));
@@ -1458,7 +1483,7 @@ PyCurses_KeyName(self,arg)
      PyObject * self;
      PyObject * arg;
 {
-  char *knp;
+  const char *knp;
   int ch;
   if (!PyArg_Parse(arg,"i",&ch))
     return NULL;
@@ -1514,7 +1539,7 @@ static PyMethodDef PyCurses_methods[] = {
 
 /* Initialization function for the module */
 
-void
+DL_EXPORT(void)
 initcurses()
 {
 	PyObject *m, *d, *v;
@@ -1527,7 +1552,7 @@ initcurses()
 	ModDict = d; /* For PyCurses_InitScr */
 
 	/* For exception curses.error */
-	PyCursesError = PyString_FromString("curses.error");
+	PyCursesError = PyErr_NewException("curses.error", NULL, NULL);
 	PyDict_SetItemString(d, "error", PyCursesError);
 
 	/* Make the version available */
@@ -1577,8 +1602,4 @@ initcurses()
 	  SetDictInt("KEY_MIN", KEY_MIN);
 	  SetDictInt("KEY_MAX", KEY_MAX);
 	}
-
-	/* Check for errors */
-	if (PyErr_Occurred())
-		Py_FatalError("can't initialize module curses");
 }
